@@ -10,6 +10,13 @@ EPOCH = datetime(1970, 1, 1, 1, 1)
 _camel_words = re.compile(r"([A-Z][a-z0-9_]+)")
 
 
+class raw_sql(str):
+    """ A custom type used for strings which shouldn't be escaped by pylytics
+    before inserting into MySQL.
+    """
+    pass
+
+
 def _camel_to_snake(s):
     """ Convert CamelCase to snake_case.
     """
@@ -33,6 +40,8 @@ def dump(value):
         return "1"
     elif value is False:
         return "0"
+    elif isinstance(value, raw_sql):
+        return value
     elif isinstance(value, str):
         return "'%s'" % value.encode("utf-8").replace("'", "''")
     elif isinstance(value, unicode):
